# coding: utf-8

from itertools import chain
import datetime
import json

from dateutil.relativedelta import relativedelta
from django.template import loader
from django.test import TestCase
from lxml import etree

from gisdo import constants as const
from gisdo import xml_helpers
from gisdo.alchemy_session import session_scope
from gisdo.algorithm.enrolled import get_22_1_index
from gisdo.algorithm.enrolled import get_22_2_index
from gisdo.algorithm.enrolled import get_22_3_index
from gisdo.algorithm.enrolled import get_22_4_index
from gisdo.algorithm.enrolled import get_22_5_index
from gisdo.algorithm.enrolled import get_22_6_index
from gisdo.algorithm.enrolled import get_22_7_index
from gisdo.algorithm.enrolled import get_22_8_index
from gisdo.algorithm.enrolled import get_22_index
from gisdo.algorithm.enrolled import get_30_1_index
from gisdo.algorithm.enrolled import get_30_2_index
from gisdo.index import ApplicationIndex
from gisdo.index import EnrolledIndex
from gisdo.index import get_queue_index
from gisdo.index import get_queue_index_collection
from gisdo.index.enrolled import Index
from gisdo.models import ReportFormRow
from gisdo.models import ScheduleSettings
from gisdo.service import DouData
from gisdo.service import TemplateView
from gisdo.tests.checklist_tests import CheckListsTestCase
from gisdo.tests.utils import TestReportService
from gisdo.tests.utils import make_decl_list
from gisdo.tests.utils import make_dou_list
from gisdo.utils import UnitHelper
from kinder.core.children.tests import factory_child
from kinder.core.declaration.models import DeclarationStatusLog
from kinder.core.declaration.models import DeclarationUnit
from kinder.core.declaration.tests import factory_declaration
from kinder.core.declaration.tests import utils as test_utils_declaration
from kinder.core.declaration_status.enum import DSS
from kinder.core.declaration_status.models import DeclarationStatus
from kinder.core.dict.models import HNE
from kinder.core.dict.models import DouType
from kinder.core.dict.models import GroupType
from kinder.core.dict.models import GroupTypeEnumerate
from kinder.core.dict.models import HealthNeed
from kinder.core.dict.models import WorkType
from kinder.core.direct.models import DRS
from kinder.core.direct.models import DirectStatus
from kinder.core.direct.proxy import DirectModel
from kinder.core.direct.tests import factory_direct
from kinder.core.group.models import Pupil
from kinder.core.group.tests import factory_group
from kinder.core.unit.models import FilialType
from kinder.core.unit.models import Unit
from kinder.core.unit.models import UnitStatus
from kinder.core.unit.tests import factory_unit

from .factory_gisdo import GisdoUnitFactory


class TestDbCheckListsTestCase(CheckListsTestCase):
    """Запускает проверки на пустой БД, только для проверки работоспособности
    тестов на проверки ФО, проверка самих данных уже собранных отчетов
     проходит в другом TestCase"""

    def setUp(self):
        self.report_form_row = self._create_data()
        self.mo = self.report_form_row.unit.get_mo()

    def _create_data(self):
        """Создаем видимость данных,
        чтобы класс сбора ФО не ругался что нет данных в МО
        """
        test_utils_declaration.load_status_dict()
        region, mo_list, dou_list = make_dou_list()
        make_decl_list(
            mo_list,
            [DSS.REGISTERED, DSS.PRIV_CONFIRMATING],
            count_decl=5
        )
        # сборка отчета
        report_service = TestReportService(region)
        report_service.collect()
        # запускаем тесты для 1 мо
        mo = mo_list[0]
        report_form_row = ReportFormRow.objects.get(
            report_id=report_service._report_form.id,
            unit=mo)
        return report_form_row


# todo: Правильно (хоть и медленней) переделать тесты на данные collect_report
class BaseGisdoReportTestCase(TestCase):
    u"""Базовый класс для тестов показателей gisdo.

    Позволяет указывать только нужные для сбора группы данных для ускорения.

    collect_only - список данных которые будут собираться:
        'applications' / 'queues' / 'enrolled' / 'capacities'
    """

    collect_only = None

    def setUp(self):
        super(BaseGisdoReportTestCase, self).setUp()
        test_utils_declaration.load_status_dict()

    def collect_report(self, region):
        u"""Строит отчет региону.

        Возвращает форму отчетности.
        """
        report_service = TestReportService(
            region,
            collect_only=self.collect_only,
        )
        report_service.collect()
        return report_service._report_form

    @staticmethod
    def get_report_form_row(unit, report_form):
        u"""Возвращает строку с данными отчета по указанной организации."""
        return report_form.reportformrow_set.get(unit=unit)


class UnitHelperTestCase(BaseGisdoReportTestCase):
    """
    Тесты для класса работы с деревом учреждений
    """
    def setUp(self):
        super(UnitHelperTestCase, self).setUp()

        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo_kazan = factory_unit.UnitMoFactory(
            name=u'Казань', parent=self.region)
        mo_kazan.save()
        mo_chelny = factory_unit.UnitMoFactory(
            name=u'Наб. Челны',parent=self.region)
        mo_chelny.save()
        mo_arsk = factory_unit.UnitMoFactory(
            name=u'Арск',parent=self.region)
        mo_arsk.save()
        self.mo_list = [mo_kazan,mo_chelny, mo_arsk]
        for mo in self.mo_list:
            for dou_i in range(0, 10):
                factory_unit.UnitDouFactory(parent=mo, name=dou_i)

    def test_mo_count(self):
        """Проверки:
        - в список мо для расчета, попадают только мо, под которыми есть доу
        :return:
        """
        self.region.save()
        self.unit_helper = UnitHelper(self.region)
        mo_count = len(list(self.unit_helper.get_report_units()))
        self.failUnlessEqual(len(self.mo_list), mo_count)


class QueueIndexTest(BaseGisdoReportTestCase):
    """
    Тесты для группы показателей "Информация о детях в очереди"
    """

    collect_only = ('queues',)

    def setUp(self):
        super(QueueIndexTest, self).setUp()

        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo = factory_unit.UnitMoFactory(parent=self.region)
        self.dou = factory_unit.UnitDouFactory(parent=mo)
        self.combi_group = factory_group.FactGroupF(
            name=u'Комбинированная',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.COMBI))
        self.decl_list = []
        for i in range(0, 10):
            child = factory_child.ChildF.create()

            decl = factory_declaration.DeclarationF.create(
                children=child, mo=mo,
                status=DeclarationStatus.get_by_code(DSS.DUL_CONFIRMATING))
            factory_declaration.DUnitF(declaration=decl, unit=self.dou)
            self.decl_list.append(decl)

    def test_181(self):
        """
        "Учитываются заявки в статусах ""Зарегистрировано""
        и ""Подтверждение льгот""
        + заявки в других статусах, у которых есть привязанные направления
        в статусах ""Подтверждено"", ""Предложено системой""
        - заявления в статусе ""Желает изменить ДОУ"".
        Дата желаемого зачисления <= 01.09.2015 (без желаемой датой зачисления),
        (01.09.текущего учебного года)
        :return:
        """

        self.assertEquals(0, len(get_queue_index_collection('18.1', self.dou)))
        self.decl_list[0].status = (
            DeclarationStatus.get_by_code(DSS.REGISTERED))
        self.decl_list[0].save()
        self.decl_list[1].status = (
            DeclarationStatus.get_by_code(DSS.PRIV_CONFIRMATING))
        self.decl_list[1].save()
        self.assertEquals(2, len(get_queue_index_collection('18.1', self.dou)))

        self.decl_list[3].status = (
            DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU))
        self.assertEquals(2, len(get_queue_index_collection('18.1', self.dou)))
        direct = factory_direct.DirectFactory(
            group=self.combi_group, declaration=self.decl_list[4])
        self.assertEquals(3, len(get_queue_index_collection('18.1', self.dou)))
        DirectModel.change_status(direct,
                                  DirectStatus.objects.get(code=DRS.CONFIRM))
        self.assertEquals(3, len(get_queue_index_collection('18.1', self.dou)))


class CapacitiesIndexTest(BaseGisdoReportTestCase):
    """
    Тесты для группы показателей "Информация о свободных местах"
    """

    collect_only = ('capacities',)

    def setUp(self):
        super(CapacitiesIndexTest, self).setUp()

        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')
        mo = factory_unit.UnitMoFactory(parent=self.region)
        self.dou = factory_unit.UnitDouFactory(parent=mo)
        self.decl_list = []
        for i in range(0, 10):
            child = factory_child.ChildF.create()

            decl = factory_declaration.DeclarationF.create(
                children=child, mo=mo,
                status=DeclarationStatus.get_by_code(DSS.DUL_CONFIRMATING))
            factory_declaration.DUnitF(declaration=decl, unit=self.dou)
            self.decl_list.append(decl)

    def test_33_index(self):
        """
        Передается численность детей, снятых с учета в течение
        **текущего календарного года** по таким причинам:

        1. Не явился (код didnt_come)
        2. Отказано в услуге (код refused)
            в случае изменения заявления со статуса
            Зарегистрировано, Желает изменить ДОУ на статус
            Отказано в услуге
        3. Архивное (код archive)
            в случае изменения заявления со статуса
            Зарегистрировано на статус Архивное
        4. Архивное (код archive)
            в случае изменения заявления со статуса
            Желает изменить ДОУ на Архивное

        При расчета показателя 33 необходимо учитывать все заявления,
        у которых дата изменения статуса заявления входит в период с 01.01.хххх,
        где хххх - текущий год.
        :return:
        """

        self.assertEquals(0, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)
        # 1
        self.decl_list[0].change_status(
            DeclarationStatus.get_by_code(DSS.DIDNT_COME))
        self.assertEquals(1, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)
        # 2
        self.assertEquals(self.decl_list[1].status.code, DSS.DUL_CONFIRMATING,
                          u'У заявления неверный статус')
        self.decl_list[1].change_status(
            DeclarationStatus.get_by_code(DSS.REFUSED))
        self.assertEquals(1, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)

        self.decl_list[2].change_status(
            DeclarationStatus.get_by_code(DSS.REGISTERED))
        self.decl_list[2].change_status(
            DeclarationStatus.get_by_code(DSS.REFUSED))
        self.assertEquals(2, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)


        self.decl_list[3].change_status(
            DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU))
        self.decl_list[3].change_status(
            DeclarationStatus.get_by_code(DSS.REFUSED))
        self.assertEquals(3, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)

        # 3
        self.decl_list[4].change_status(
            DeclarationStatus.get_by_code(DSS.ARCHIVE))
        self.assertEquals(3, get_queue_index("33", self.dou),
                          u'Кол-во не должно изменится, тк не выполнено условие'
                          u'что статус был "Зарегистрировано"' )

        self.decl_list[4].change_status(
            DeclarationStatus.get_by_code(DSS.REGISTERED))
        self.decl_list[4].change_status(
            DeclarationStatus.get_by_code(DSS.ARCHIVE))
        self.assertEquals(4, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)
        # 4
        self.decl_list[5].change_status(
            DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU))
        self.decl_list[5].change_status(
            DeclarationStatus.get_by_code(DSS.ARCHIVE))
        self.assertEquals(5, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)

        # проверка на дату изменения
        DeclarationStatusLog.objects.filter(
            status=DeclarationStatus.get_by_code(DSS.ARCHIVE)).update(
            datetime=datetime.datetime.now() - relativedelta(years=1))
        self.assertEquals(3, get_queue_index("33", self.dou),
                          u'Ошибка по организации %s' % self.dou.name)


class ApplicationIndexTest(BaseGisdoReportTestCase):
    """
    Тесты для группы показателей "Информация о заявлениях"
    """

    collect_only = ('applications',)

    def test_sum_age_slice_equal_sum_delivery_slice(self):
        """
        Проверяем, чтобы сумма количества заявлений в разрезе возрастных
        категорий была равно сумме количества заявлений в разрезе типа подачи
        """

        units = Unit.objects.filter(kind__id=4)

        with session_scope() as session:
            for dou in units:
                index = ApplicationIndex(dou, session)
                sum_in_age_slice = 0
                for age in const.AGE_CATEGORIES_FULL.values():
                    sum_in_age_slice += index.get_count(age_range=age)
                sum_in_delivery_slice = 0
                sum_in_delivery_slice += (index.get_count(
                    index_type=ApplicationIndex.COUNT_BY_DELIVERY,
                    age_range=age))
                sum_in_delivery_slice += (index.get_count(
                    index_type=ApplicationIndex.COUNT_BY_DELIVERY,
                    portal=1,
                    age_range=age))
                self.assertEquals(sum_in_age_slice, sum_in_delivery_slice,
                                  u'Ошибка по организации %s' % dou.name)


class EnrolledIndexTest(BaseGisdoReportTestCase):
    """
    Тесты для группы показателей "Информация о зачисленных детях"
    Важно! У класса gisdo.index.enrolled.Index cписок групп кешируется,
    поэтому при изменение группы неоходимо создавать новый инстанс индекса
    """

    collect_only = ('enrolled',)

    def setUp(self):
        super(EnrolledIndexTest, self).setUp()

        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')

        self.mo = factory_unit.UnitMoFactory(parent=self.region)

        self.dou = factory_unit.UnitDouFactory(parent=self.mo)
        GisdoUnitFactory(unit=self.dou)

        self.filial = factory_unit.FilialFactory(parent=self.mo)
        GisdoUnitFactory(unit=self.filial)
        factory_unit.FilialDataFactory(head=self.dou, filial=self.filial)

        self.child_list = []
        for i in range(0, 10):
            self.child_list.append(factory_child.ChildF.create())

        self.comp_group = factory_group.FactGroupF(
            name=u'Компенсирующая',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.COMP),
        )
        self.health_group = factory_group.FactGroupF(
            name=u'Оздоровительная',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.HEALTH),
        )
        self.combi_group = factory_group.FactGroupF(
            name=u'Комбинированная',
            unit=self.dou,
            type=GroupType.get_by_code(GroupTypeEnumerate.COMBI),
        )
        self.fact_group = factory_group.FactGroupF(
            name=u'Фактическая',
            unit=self.dou,
        )
        self.plan_group = factory_group.PlanGroupF(
            name=u'Плановая',
            unit=self.dou,
            room=self.fact_group.room,
        )
        self.filial_fact_group = factory_group.FactGroupF(
            name=u'Фактическая филиала',
            unit=self.filial,
        )
        self.filial_plan_group = factory_group.PlanGroupF(
            name=u'Плановая филиала',
            unit=self.filial,
            room=self.filial_fact_group.room,
        )
        self.short_wt_fact_group = factory_group.FactGroupF(
            name=u'Фактическая кратковременного пребывания',
            work_type=factory_group.WorkTypeF(code=WorkType.SHORT),
            unit=self.dou,
        )
        self.short_wt_plan_group = factory_group.PlanGroupF(
            name=u'Плановая кратковременного пребывания',
            work_type=factory_group.WorkTypeF(code=WorkType.SHORT),
            unit=self.dou,
            room=self.short_wt_fact_group.room
        )
        self.short_wt_filial_fact_group = factory_group.FactGroupF(
            name=u'Фактическая кратковременного пребывания филиала',
            work_type=factory_group.WorkTypeF(code=WorkType.SHORT),
            unit=self.filial
        )
        self.short_wt_filial_plan_group = factory_group.FactGroupF(
            name=u'Плановая кратковременного пребывания филиала',
            work_type=factory_group.WorkTypeF(code=WorkType.SHORT),
            unit=self.filial,
            room=self.short_wt_filial_fact_group.room
        )

    def test_enrolled_index_fact_priority_main_to_filial(self):
        u"""В случае если ребенок числился в факт. группе в головной
        организации, а в плановой группе был переведен в филиал или корпус,
        такого ребенка необходимо учитывать в расчете 1 раз по факт. группе
        """
        enrolled_index = Index(self.dou)

        factory_group.PupilF(
            grup=self.fact_group, children=self.child_list[0])
        factory_group.PupilF(
            grup=self.filial_plan_group, children=self.child_list[0])

        self.assertEquals(1, len(list(enrolled_index(force=True))))

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)
        groups = list(
            chain(*(building['groups'] for building in dou_data.buildings))
        )

        main_enrolled = filter(
            lambda g: g['id'] == self.fact_group.id, groups
        )[0]['enrolled']
        self.assertEquals(1, main_enrolled)

        total_enrolled = sum(group['enrolled'] for group in groups)
        self.assertEquals(1, total_enrolled)

    def test_enrolled_index_fact_priority_filial_to_main(self):
        u"""В случае если ребенок числился в факт. группе в филиальной
        организации, а в плановой группе был переведен в головную организацию,
        такого ребенка необходимо учитывать в расчете 1 раз по факт. группе
        """
        enrolled_index = Index(self.dou)

        factory_group.PupilF(
            grup=self.plan_group, children=self.child_list[0])
        factory_group.PupilF(
            grup=self.filial_fact_group, children=self.child_list[0])

        self.assertEquals(1, len(list(enrolled_index(force=True))))

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)
        groups = list(
            chain(*(building['groups'] for building in dou_data.buildings))
        )

        filial_enrolled = filter(
            lambda g: g['id'] == self.filial_fact_group.id, groups
        )[0]['enrolled']
        self.assertEquals(1, filial_enrolled)

        total_enrolled = sum(group['enrolled'] for group in groups)
        self.assertEquals(1, total_enrolled)

    def test_enrolled_includes_filials(self):
        """В список зачисленных по ДОО попадают также дети связанных филиалов
        """
        enrolled_index = Index(self.dou)

        factory_group.PupilF(
            grup=self.fact_group, children=self.child_list[0])
        factory_group.PupilF(
            grup=self.filial_fact_group, children=self.child_list[1])

        self.assertEquals(2, len(list(enrolled_index(force=True))))

    def test_enrolled_index_unique_different_groups(self):
        """В список зачисленных ребенок должен попадать 1 раз
        в случае если он зачислен в две разные группы
        """
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[0])

        self.assertEquals(len(list(enrolled_index(force=True))), 1)

    def test_enrolled_index_unique_same_group(self):
        """В список зачисленных ребенок должен попадать 1 раз
        в случае если он зачислен в одну и ту же группу
        """
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])

        self.assertEquals(len(list(enrolled_index(force=True))), 1)

    def test_enrolled_index_fact_priority(self):
        """В список зачисленных ребенок приоритетно попадает
        по фактическим группам
        """
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.plan_group, children=self.child_list[0])
        fact_pupil_0 = factory_group.PupilF(
            grup=self.fact_group, children=self.child_list[0])

        fact_pupil_1 = factory_group.PupilF(
            grup=self.fact_group, children=self.child_list[1])
        factory_group.PupilF(grup=self.plan_group, children=self.child_list[1])

        pupils_fact_group_filtered_count = Pupil.objects.filter(
            enrolled_index.pupil_filter
        ).filter(
            id__in=(fact_pupil_0.id, fact_pupil_1.id)
        ).count()

        self.assertEquals(pupils_fact_group_filtered_count, 2)

    def test_191(self):
        """
        Количество детей в текущих группах.
         Временно зачисленные учитываются, временно отчисленные нет.
        :return:
        """
        enrolled_index = Index(self.dou)
        index_19 = list(enrolled_index(force=True))
        self.assertEquals(
            0, len(index_19), u'Показатель 19 не 0')

        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        factory_group.PupilF(grup=self.health_group, temporary=True,
                             children=self.child_list[2])

        self.assertEquals(3, len(list(enrolled_index(force=True))))

    def test_20_8(self):
        """
        Показатель 20.8

        То же что и 20.5-й, но остаются только те заявления
        дети в которых зачислены в ДОО с типом:
            НЕ муниципальная/федеральная/государственная,
        и планируют перейти в ДОО с типом:
        муниципальная/федеральная/государственная.
        """
        self.government_dou = factory_unit.UnitDouFactory(
            parent=self.mo,
            dou_type=DouType.objects.get(code=DouType.MUNICIPAL)
        )
        GisdoUnitFactory(unit=self.government_dou)
        self.private_dou = factory_unit.UnitDouFactory(
            parent=self.mo,
            dou_type=DouType.objects.get(code=DouType.IP_WITH_LICENSE)
        )
        GisdoUnitFactory(unit=self.private_dou)

        self.government_group = factory_group.FactGroupF(
            unit=self.government_dou,
        )
        self.private_group = factory_group.FactGroupF(unit=self.private_dou)

        # учится в гос. и имеет заявление в частный (не подходит)
        factory_group.PupilF(
            grup=self.government_group, children=self.child_list[0])
        DeclarationUnit.objects.create(
            declaration=factory_declaration.DeclarationF(
                children=self.child_list[1],
                mo=self.mo,
                status=DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU),
            ),
            unit=self.private_dou,
        )

        # учится в гос. и имеет заявление в гос. (не подходит)
        factory_group.PupilF(
            grup=self.government_group, children=self.child_list[1])
        DeclarationUnit.objects.create(
            declaration=factory_declaration.DeclarationF(
                children=self.child_list[1],
                mo=self.mo,
                status=DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU),
            ),
            unit=self.government_dou,
        )

        # учится в частном и имеет заявление в гос (подходит)
        factory_group.PupilF(
            grup=self.private_group, children=self.child_list[2])
        DeclarationUnit.objects.create(
            declaration=factory_declaration.DeclarationF(
                children=self.child_list[2],
                mo=self.mo,
                status=DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU),
            ),
            unit=self.government_dou,
        )

        # учится в частном и имеет заявление в частный (не подходит)
        factory_group.PupilF(
            grup=self.government_group, children=self.child_list[3])
        DeclarationUnit.objects.create(
            declaration=factory_declaration.DeclarationF(
                children=self.child_list[3],
                mo=self.mo,
                status=DeclarationStatus.get_by_code(DSS.WANT_CHANGE_DOU),
            ),
            unit=self.private_dou,
        )

        report_form = self.collect_report(self.region)
        gov_report_form_row = self.get_report_form_row(
            self.government_dou, report_form
        )
        priv_report_form_row = self.get_report_form_row(
            self.private_dou, report_form
        )

        gov_enrolled = json.loads(gov_report_form_row.enrolled)
        priv_enrolled = json.loads(priv_report_form_row.enrolled)

        # итого по гос саду 0
        self.assertEquals(
            0,
            sum(gov_enrolled['20.8'].values()),
            u'Ошибка в показателе 20_8 по организации %s'
            % self.government_dou.name
        )

        # итого по частному саду 1
        self.assertEquals(
            1,
            sum(priv_enrolled['20.8'].values()),
            u'Ошибка в показателе 20_8 по организации %s'
            % self.private_dou.name
        )

    def test_22(self):
        """
        Общая численность детей, зачисленных в группы для детей с ОВЗ.
        Количество детей в текущих группах с направленностью "Комбинированноая"
         или/и "Компенсирующая".Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_index(enrolled_index)), u'Показатель 22 не 0')

        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        factory_group.PupilF(grup=self.health_group,
                             children=self.child_list[2])

        self.assertEquals(2, len(get_22_index(enrolled_index)))

    def test_221(self):
        """
        Передается общая численность детей с ОВЗ, зачисленных в
        группы компенсирующей и комбинированной направленности для детей с
        нарушением слуха.
        Количество детей в группе "Компенсирующая" с овз = с нарушением слуха
        (deafness, hardofhearing). Дети в комбинированных группа
        с овз =(deafness, hardofhearing) у которых в карточке ребенка есть овз
        (если у ребенка выбран ОВЗ оздор. типа эти дети попадают в показатель
        только при наличии заполненного поля Документ, подтверждающий ОВЗ)
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_1_index(enrolled_index)), u'Показатель 22.1 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.DEAFNESS)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_1_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.HARDOFHEARTING)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.HARDOFHEARTING)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_1_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_1_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_1_index(enrolled_index)))

    def test_222(self):
        """
        Количество детей в группе "Компенсирующая" с овз = с нарушением речи
        (speach, phonetics). Дети в комбинированных группа
        с овз =(speach, phonetics) у которых в карточке ребенка есть овз
        (если у ребенка выбран ОВЗ оздор. типа эти дети попадают в показатель
        только при наличии заполненного поля Документ, подтверждающий ОВЗ)
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_2_index(enrolled_index)), u'Показатель 22.2 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.SPEACH)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_2_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.PHONETICS)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.PHONETICS)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_2_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_2_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_2_index(enrolled_index)))

    def test_223(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей с нарушением зрения(blindness, AMAUROSIS).
        Дети в комбинированных группа с овз = (blindness, AMAUROSIS)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_3_index(enrolled_index)), u'Показатель 22.3 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.BLINDNESS)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_3_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.AMAUROSIS)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.AMAUROSIS)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_3_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_3_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_3_index(enrolled_index)))

    def test_224(self):
        """
        Количество детей в группе "Компенсирующая" с овз = с нарушением интел.
        (backwardnesslight, backwardnesshard). Дети в комбинированных группа
        с овз =(backwardnesslight, backwardnesshard)
        у которых в карточке ребенка есть овз
        (если у ребенка выбран ОВЗ оздор. типа эти дети попадают в показатель
        только при наличии заполненного поля Документ, подтверждающий ОВЗ)
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются.
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_4_index(enrolled_index)), u'Показатель 22.4 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.BACKHARD)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_4_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.BACKLIGHT)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.BACKLIGHT)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_4_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_4_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_4_index(enrolled_index)))

    def test_225(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
         для детей с задержкой психического развития.(backwardness).
        Дети в комбинированных группа с овз = (backwardness)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_5_index(enrolled_index)), u'Показатель 22.5 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.BACK)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_5_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.BACK)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.BACK)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_5_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_5_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_5_index(enrolled_index)))

    def test_226(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей с нарушением опорно-двигательного аппарата (disablement)
        Дети в комбинированных группа с овз = (disablement)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_6_index(enrolled_index)), u'Показатель 22.6 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.DISABLEMENT)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_6_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.DISABLEMENT)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.DISABLEMENT)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_6_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_6_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_6_index(enrolled_index)))

    def test_227(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей со сложным дефектом(invalidity).
        Дети в комбинированных группа с овз = (invalidity)
        у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_7_index(enrolled_index)), u'Показатель 22.7 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.INVALIDITY)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_7_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.combi_group.health_need = HealthNeed.get_by_code(
            HNE.INVALIDITY)
        self.combi_group.save()
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.INVALIDITY)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_7_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_7_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_7_index(enrolled_index)))

    def test_228(self):
        """
        Количество детей в группе "Компенсирующая" с овз =
        для детей с иными ограниченными возможностями здоровья.
        Дети в комбинированных группа с овз, у которых в карточке ребенка овз (
        если у ребенка выбран ОВЗ оздор. типа ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ).
        постоянно зачисленные, временно зачисленные в фактическую группу.
        Временно отчисленные не учитываются
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_22_8_index(enrolled_index)), u'Показатель 22.8 не 0')

        enrolled_index = Index(self.dou)
        self.comp_group.health_need = HealthNeed.get_by_code(HNE.RESTRICTION)
        self.comp_group.save()
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(1, len(get_22_8_index(enrolled_index)))

        enrolled_index = Index(self.dou)
        self.child_list[1].health_need = HealthNeed.get_by_code(
            HNE.DEAFNESS)
        self.child_list[1].save()
        factory_group.PupilF(grup=self.combi_group, children=self.child_list[1])
        self.assertEquals(2, len(get_22_8_index(enrolled_index)))

        # для этих ОВЗ только при наличие документа
        enrolled_index = Index(self.dou)
        self.child_list[2].health_need = HealthNeed.get_by_code(HNE.SICK)
        self.child_list[2].save()
        factory_group.PupilF(grup=self.combi_group,
                             children=self.child_list[2])
        self.assertEquals(2, len(get_22_8_index(enrolled_index)))
        self.child_list[2].health_need_confirmation = u'Док'
        self.child_list[2].save()
        self.assertEquals(3, len(get_22_8_index(enrolled_index)))

    def test_27(self):
        u"""
        Общая численность детей, зачисленных в группы кратковременного
        пребывания детей.

        Передается общая численность детей, зачисленных в группы
        кратковременного пребывания.
        """

        # ребенок должен попасть только один раз
        factory_group.PupilF(
            grup=self.short_wt_fact_group, children=self.child_list[0])
        factory_group.PupilF(
            grup=self.short_wt_plan_group, children=self.child_list[0])
        factory_group.PupilF(
            grup=self.short_wt_filial_fact_group, children=self.child_list[0])
        factory_group.PupilF(
            grup=self.short_wt_filial_plan_group, children=self.child_list[0])

        # попадает по фактической
        factory_group.PupilF(
            grup=self.short_wt_fact_group, children=self.child_list[1])

        # попадает по плановой
        factory_group.PupilF(
            grup=self.short_wt_plan_group, children=self.child_list[2])

        # попадает из фактической филиала
        factory_group.PupilF(
            grup=self.short_wt_filial_fact_group, children=self.child_list[3])

        # попадает из плановой филиала
        factory_group.PupilF(
            grup=self.short_wt_filial_fact_group, children=self.child_list[4])

        dou_report_form_row = self.get_report_form_row(
            self.dou, self.collect_report(self.region)
        )
        enrolled = json.loads(dou_report_form_row.enrolled)

        # итого 5
        self.assertEquals(
            5,
            sum(enrolled['27'].values()),
            u'Ошибка в показателе 27 по организации %s' % self.dou.name
        )

    def test_28(self):
        u"""
        Общая численность детей, посещающих ДОО в режиме
        кратковременного пребывания.
        Описание: Передается общая численность детей, посещающих ДОО в
        режиме кратковременного пребывания вне зависимости от режима работы
        группы.
        """

        # попадает по фактической по режиму работы
        factory_group.PupilF(
            grup=self.short_wt_fact_group, children=self.child_list[0])

        # попадает по фактической по режиму работы
        # (особый режим также кратковременный)
        factory_group.PupilF(
            grup=self.short_wt_fact_group,
            children=self.child_list[1],
            actual_work_type=factory_group.WorkTypeF(code=WorkType.SHORT)
        )

        # попадает по плановой по режиму работы
        factory_group.PupilF(
            grup=self.short_wt_plan_group, children=self.child_list[2])

        # должен был попасть по фактической по режиму работы
        # но исключается, т.к. указан другой особый режим пребывания
        factory_group.PupilF(
            grup=self.short_wt_fact_group,
            children=self.child_list[3],
            actual_work_type=factory_group.WorkTypeF(code=WorkType.ALLDAY)
        )

        # не должен был попасть по фактической по режиму работы
        # но попадает, т.к. указан кратковременный режим пребывания
        factory_group.PupilF(
            grup=self.fact_group,
            children=self.child_list[4],
            actual_work_type=factory_group.WorkTypeF(code=WorkType.SHORT)
        )

        dou_report_form_row = self.get_report_form_row(
            self.dou, self.collect_report(self.region)
        )
        enrolled = json.loads(dou_report_form_row.enrolled)

        # итого 4
        self.assertEquals(
            4,
            sum(enrolled['28'].values()),
            u'Ошибка в показателе 28 по организации %s' % self.dou.name
        )

    def test_301(self):
        """
        Численность детей-инвалидов, имеющих ограниченные возможности здоровья.

        Передается численность детей-инвалидов, зачисленных в ДОО,
        имеющих ограниченние возможностей здоровья, посещающих группы любой
        направленности (т.е. считаем всех детей с чек-боксов ребенок-инвалид
        во всех группах).
        При этом, если у ребенка выбран ОВЗ типа "часто болеющие",
        "с туберкулезной интоксикацией",
        "другие, с необходимостью комплексных оздоровительных мероприятий" ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ.
        :return:
        """

        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_30_1_index(enrolled_index)), u'Показатель 30.1 не 0')

        # В показателе только дети с инвалидностью
        for ch in self.child_list:
            ch.is_invalid = True
            ch.save()
        # зачислим первого ребенка в компенс. группу
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(
            1, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в компенсирующих группах с '
            u'инвалидностью')

        # зачислим второго ребенка в оздоровительную группу
        enrolled_index = Index(self.dou)
        child = self.child_list[1]
        factory_group.PupilF(grup=self.health_group,
                             children=child)
        self.assertEquals(
            1, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            1, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            2, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        # в группах с типом отличным от оздор. и компенс.
        # проверяется наличие ОВЗ
        enrolled_index = Index(self.dou)
        child = self.child_list[2]
        factory_group.PupilF(grup=self.combi_group, children=child)
        # С ОВЗ для оздоровитеьных групп, но с типом не оздоровительная,
        # ребенок не попадет в показатель
        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            2, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            3, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп, '
            u'либо с подтверждением')

        enrolled_index = Index(self.dou)
        child = self.child_list[3]
        factory_group.PupilF(grup=self.combi_group, children=child)
        child.health_need = HealthNeed.get_by_code(HNE.DEAFNESS)
        child.save()
        self.assertEquals(
            4, len(get_30_1_index(enrolled_index)),
            u'Показатель 30.1 учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

    def test_302(self):
        """
        Численность детей, не имеющих инвалидности,
        но имеющих ограниченные возможности здоровья.
        Передается численность детей, зачисленных в ДОО,
        не имеющих инвалидности, но имеющих ограниченныых возможностей
        здоровья, посещающих группы любой направленности.
        При этом, если у ребенка выбран ОВЗ типа "часто болеющие",
        "с туберкулезной интоксикацией",
        "другие, с необходимостью комплексных оздоровительных мероприятий" ,
        эти дети попадают в показатель только при наличии заполненного поля
        Документ, подтверждающий ОВЗ
        Отличие от тестов на 30.1 только отсутсвием признака инвалид детей
        :return:
        """
        enrolled_index = Index(self.dou)
        self.assertEquals(
            0, len(get_30_2_index(enrolled_index)), u'Показатель 30.2не 0')

        # зачислим первого ребенка в компенс. группу
        enrolled_index = Index(self.dou)
        factory_group.PupilF(grup=self.comp_group, children=self.child_list[0])
        self.assertEquals(
            1, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в компенсирующих группах с '
            u'инвалидностью')

        # зачислим второго ребенка в оздоровительную группу
        enrolled_index = Index(self.dou)
        child = self.child_list[1]
        factory_group.PupilF(grup=self.health_group,
                             children=child)
        self.assertEquals(
            1, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            1, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            2, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей в оздор. группах с '
            u'инвалидностью и ОВЗ, только при наличие документа')

        # в группах с типом отличным от оздор. и компенс.
        # проверяется наличие ОВЗ
        enrolled_index = Index(self.dou)
        child = self.child_list[2]
        factory_group.PupilF(grup=self.combi_group, children=child)
        # С ОВЗ для оздоровитеьных групп, но с типом не оздоровительная,
        # ребенок не попадет в показатель
        child.health_need = HealthNeed.get_by_code(HNE.PHTHISIS)
        child.save()
        self.assertEquals(
            2, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

        child.health_need_confirmation = u'Док'
        child.save()
        self.assertEquals(
            3, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп, '
            u'либо с подтверждением')

        enrolled_index = Index(self.dou)
        child = self.child_list[3]
        factory_group.PupilF(grup=self.combi_group, children=child)
        child.health_need = HealthNeed.get_by_code(HNE.DEAFNESS)
        child.save()
        self.assertEquals(
            4, len(get_30_2_index(enrolled_index)),
            u'Показатель 30.2учитывает детей не в оздор. и не комп. группах с '
            u'инвалидностью и ОВЗ, отличным от ОВЗ для оздор. групп')

    def test_sum_disabled_group_equals_sum_correct_and_health_groups(self):
        """
        Сумма показателей "Количества детей в группах для детей с
        ограниченными возможностями" в разрезе возрастных
        категорий должно равняться сумме "Количества детей
         в оздоровительных группах" и
        "Количество детей в компенсирующих группах
        """

        units = Unit.objects.filter(kind__id=4)

        with session_scope() as session:
            for dou in units:
                index = EnrolledIndex(dou, session)
                sum_in_disabled_group_age_slice = 0
                for age in const.AGE_CATEGORIES_FULL.values():
                    sum_in_disabled_group_age_slice += index.get_count(
                        age_range=age,
                        index_type=EnrolledIndex.COUNT_DISABLED).count()
                sum_in_correct_and_health_groups = 0
                sum_in_correct_and_health_groups += index.get_count(
                    index_type=EnrolledIndex.COUNT_CORRECT).count()
                sum_in_correct_and_health_groups += index.get_count(
                    index_type=EnrolledIndex.COUNT_HEALTH).count()

                self.assertEqual(sum_in_disabled_group_age_slice,
                                 sum_in_correct_and_health_groups,
                                 u'Ошибка по организации %s' % dou.name)


class EmptyBuildingsTest(BaseGisdoReportTestCase):
    u"""Тестирование случаев отсутствия в филиале/головной организации
    учеников/групп.

    Во всех тестах также проверяется соответствие
    organization/num_building и числа переданных buildings.
    """

    def setUp(self):
        super(EmptyBuildingsTest, self).setUp()

        test_utils_declaration.load_status_dict()
        self.region = factory_unit.UnitRFactory(name=u'Татарстан')

        self.mo = factory_unit.UnitMoFactory(parent=self.region)

        self.dou = factory_unit.UnitDouFactory(parent=self.mo)
        GisdoUnitFactory(unit=self.dou)

        self.filial = factory_unit.FilialFactory(parent=self.mo)
        GisdoUnitFactory(unit=self.filial)
        factory_unit.FilialDataFactory(head=self.dou, filial=self.filial)

        self.child = factory_child.ChildF.create()

    def test_empty_buildings_no_filial_groups(self):
        u"""
        Условие:
            в головной организации есть дети
            в филиале нет детей (нет групп)

        Отправляемые значения:
            один building(головной)
            значение free_space и показатели 31.x считаются

        Статус "функционирует" можно не проверять
        """

        self.group = factory_group.FactGroupF(unit=self.dou)
        factory_group.PupilF(grup=self.group, children=self.child)

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)
        groups = list(
            chain(*(building['groups'] for building in buildings))
        )

        self.assertEqual(len(buildings), 1)
        self.assertEqual(dou_data.organization['num_building'], 1)

        self.assertEqual(dou_data.organization['status'], xml_helpers.WORK)

        total_groups_free_space = 0
        for group in groups:
            self.assertNotEqual(group['free_space'], 0)
            total_groups_free_space += group['free_space']

        enrolled_index = Index(self.dou)
        self.assertEqual(
            total_groups_free_space,
            sum(enrolled_index.free_space_in_unit(force=True).values())
        )

    def test_empty_buildings_no_main_groups(self):
        u"""
        Условие:
            в головной организации нет детей (нет групп)
            в филиале есть дети

        Отправляемые значения:
            один building (филиал)
            значение free_space и показатели 31.x считаются

        статус "функционирует" зданий можно не проверять
        """
        self.filial_group = factory_group.FactGroupF(unit=self.filial)
        factory_group.PupilF(grup=self.filial_group, children=self.child)

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)
        groups = list(
            chain(*(building['groups'] for building in buildings))
        )

        self.assertEqual(len(buildings), 1)
        self.assertEqual(dou_data.organization['num_building'], 1)

        self.assertEqual(dou_data.organization['status'], xml_helpers.WORK)

        total_groups_free_space = 0
        for group in groups:
            self.assertNotEqual(group['free_space'], 0)
            total_groups_free_space += group['free_space']

        enrolled_index = Index(self.dou)
        self.assertEqual(
            total_groups_free_space,
            sum(enrolled_index.free_space_in_unit(force=True).values())
        )

    def test_empty_buildings_no_main_pupils(self):
        u"""
        Условие:
            В головной организации нет детей (есть группы)
            в филиале есть дети

        Отправляемые значения:
            два building (филиал и головное)
            значение free_space и показатели 31.x считаются

        статус "функционирует" зданий можно не проверять
        """
        self.group = factory_group.FactGroupF(unit=self.dou)
        self.filial_group = factory_group.FactGroupF(unit=self.filial)
        factory_group.PupilF(grup=self.filial_group, children=self.child)

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)
        groups = list(
            chain(*(building['groups'] for building in buildings))
        )

        self.assertEqual(len(buildings), 2)
        self.assertEqual(dou_data.organization['num_building'], 2)

        self.assertEqual(dou_data.organization['status'], xml_helpers.WORK)

        total_groups_free_space = 0
        for group in groups:
            self.assertNotEqual(group['free_space'], 0)
            total_groups_free_space += group['free_space']

        enrolled_index = Index(self.dou)
        self.assertEqual(
            total_groups_free_space,
            sum(enrolled_index.free_space_in_unit(force=True).values())
        )

    def test_empty_buildings_no_filial_pupils(self):
        u"""
        Условие:
            в головной организации есть дети
            в филиале нет детей (есть группы)

        Отправляемые значения:
            два building (филиал и головное)
            значение free_space и показатели 31.x считаются

        статус "функционирует" зданий можно не проверять
        """
        self.group = factory_group.FactGroupF(unit=self.dou)
        self.filial_group = factory_group.FactGroupF(unit=self.filial)
        factory_group.PupilF(grup=self.group, children=self.child)

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)
        groups = list(
            chain(*(building['groups'] for building in buildings))
        )

        self.assertEqual(len(buildings), 2)
        self.assertEqual(dou_data.organization['num_building'], 2)

        self.assertEqual(dou_data.organization['status'], xml_helpers.WORK)

        total_groups_free_space = 0
        for group in groups:
            self.assertNotEqual(group['free_space'], 0)
            total_groups_free_space += group['free_space']

        enrolled_index = Index(self.dou)
        self.assertEqual(
            total_groups_free_space,
            sum(enrolled_index.free_space_in_unit(force=True).values())
        )

    def test_empty_buildings_no_main_groups_no_filial_pupils_not_func(self):
        u"""
        Условие:
            в головной организации нет детей (нет групп)
            в филиале нет детей (есть группы)
            головная организация не функционирует

        Отправляемые значения:
            один building (филиал)
            значение free_space и показатели 31.x приравниваем 0

        статус смотрим у головной организации
        """
        self.filial_group = factory_group.FactGroupF(unit=self.filial)
        self.dou.status = UnitStatus.RECONSTRUCTION

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)
        groups = list(
            chain(*(building['groups'] for building in buildings))
        )

        self.assertEqual(len(buildings), 1)
        self.assertEqual(dou_data.organization['num_building'], 1)

        self.assertEqual(
            dou_data.organization['status'],
            xml_helpers.RECONSTRUCTION
        )

        total_groups_free_space = 0
        for group in groups:
            self.assertEqual(group['free_space'], 0)
            total_groups_free_space += group['free_space']

        enrolled_index = Index(self.dou)
        self.assertEqual(
            total_groups_free_space,
            sum(enrolled_index.free_space_in_unit(force=True).values())
        )

    def test_empty_buildings_no_main_groups_no_filial_pupils_func(self):
        u"""
        Условие:
            в головной организации нет детей (нет групп)
            в филиале нет детей (есть группы)
            головная организация функционирует

        Отправляемые значения:
            один building (филиал)
            значение free_space и показатели 31.x считаются

        статус передаем "Контингент отсутствует"
        """
        self.filial_group = factory_group.FactGroupF(unit=self.filial)
        self.dou.status = UnitStatus.NONE

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)
        groups = list(
            chain(*(building['groups'] for building in buildings))
        )

        self.assertEqual(len(buildings), 1)
        self.assertEqual(dou_data.organization['num_building'], 1)

        self.assertEqual(
            dou_data.organization['status'],
            xml_helpers.NO_CONTINGENT
        )

        total_groups_free_space = 0
        for group in groups:
            self.assertEqual(group['free_space'], self.filial_group.max_count)
            total_groups_free_space += group['free_space']

        enrolled_index = Index(self.dou)
        self.assertEqual(
            total_groups_free_space,
            sum(enrolled_index.free_space_in_unit(force=True).values())
        )

    def test_empty_buildings_no_main_groups_no_filial_groups_not_func(self):
        u"""
        Условие:
            в головной организации нет детей (нет групп)
            в филиале нет детей (нет групп)

        Отправляемые значения:
            building нет

        статус смотрим у головной организации
        если функционирует - передаем "Контингент отсутствует"
        в начале смотрим статус который выставлен
        """
        self.dou.status = UnitStatus.RECONSTRUCTION

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)

        self.assertEqual(len(buildings), 0)
        self.assertEqual(dou_data.organization['num_building'], 0)

        self.assertEqual(
            dou_data.organization['status'],
            xml_helpers.RECONSTRUCTION
        )

    def test_empty_buildings_no_main_groups_no_filial_groups_func(self):
        u"""
        Условие:
            в головной организации нет детей (нет групп)
            в филиале нет детей (нет групп)

        Отправляемые значения:
            building нет

        статус смотрим у головной организации
        если функционирует - передаем "Контингент отсутствует"
        в начале смотрим статус который выставлен
        """
        self.dou.status = UnitStatus.NONE

        report_service = TestReportService(self.region)
        report_service.collect()
        dou_data = DouData(self.dou, report_service._report_form)

        buildings = list(dou_data.buildings)

        self.assertEqual(len(buildings), 0)
        self.assertEqual(dou_data.organization['num_building'], 0)

        self.assertEqual(
            dou_data.organization['status'],
            xml_helpers.NO_CONTINGENT
        )


class XmlResultsTest(BaseGisdoReportTestCase):
    u"""Тестирует данные итоговой XML отчета."""

    @staticmethod
    def _get_report_xml_string(mo, report_form):
        u"""Возвращает строку с XML данными отчета по конкретному МО"""
        document = TemplateView(
            mo, report_form.date,
            ScheduleSettings.get_settings(),
            UnitHelper(report_form.unit),
            report_form,
        )

        return loader.render_to_string(
            'ws/gisdo_request_template_5_0.xml',
            {'doc': document}
        ).encode('utf-8')

    def setUp(self):
        super(XmlResultsTest, self).setUp()

        test_utils_declaration.load_status_dict()
        self.max_doo_region = factory_unit.UnitRFactory(
            max_desired_dou=55,
        )

        self.no_max_doo_region = factory_unit.UnitRFactory()

        self.mo_1 = factory_unit.UnitMoFactory(
            parent=self.max_doo_region,
            max_desired_dou=5,
        )
        GisdoUnitFactory(unit=factory_unit.UnitDouFactory(parent=self.mo_1))

        self.mo_2 = factory_unit.UnitMoFactory(parent=self.max_doo_region)
        GisdoUnitFactory(unit=factory_unit.UnitDouFactory(parent=self.mo_2))

        self.mo_3 = factory_unit.UnitMoFactory(parent=self.no_max_doo_region)
        self.good_dou = factory_unit.UnitDouFactory(parent=self.mo_3)
        self.closed_dou = factory_unit.UnitDouFactory(
            parent=self.mo_3,
            status=UnitStatus.CLOSED,
        )
        self.filial_dou = factory_unit.UnitDouFactory(
            parent=self.mo_3,
            is_filial=FilialType.FILIAL,
        )
        GisdoUnitFactory(unit=self.good_dou)
        GisdoUnitFactory(unit=self.closed_dou)
        GisdoUnitFactory(unit=self.filial_dou)

    def test_max_doo(self):
        max_doo_region_form = self.collect_report(self.max_doo_region)
        no_max_doo_region_form = self.collect_report(self.no_max_doo_region)

        test_results_map = {
            # результат из max_desired_dou самого МО
            (self.mo_1, max_doo_region_form): 5,
            # результат из max_desired_dou РЕГИОНА
            (self.mo_2, max_doo_region_form): 55,
            # в МО и РЕГИОНЕ не указан
            # берем по числу ДОО (закрытый ДОО и филиал не попадают)
            (self.mo_3, no_max_doo_region_form): 1,
        }

        for (mo, report_form), max_doo in test_results_map.iteritems():
            root = etree.fromstring(
                self._get_report_xml_string(mo, report_form)
            )

            xml_max_doo = root.xpath(
                "//*[local-name()='municipality']"
            )[0].get('max_doo')

            self.assertEqual(max_doo, int(xml_max_doo))
